@@ -21,6 +21,8 @@ from offlineimap import imaplib, imaputil, imapserver, repository, folder, mbnam
 import re, os, os.path, offlineimap, sys
 from ConfigParser import ConfigParser
 
+# imaplib.Debug = 5
+
 ui = offlineimap.ui.TTY.TTYUI()
 ui.init_banner()
 
@@ -101,7 +103,18 @@ def syncitall():
             statusfolder = statusrepos.getfolder(remotefolder.getvisiblename())
             statusfolder.cachemessagelist()
 
+            #
+
             if not statusfolder.isnewfolder():
+                # Delete local copies of remote messages.  This way,
+                # if a message's flag is modified locally but it has been
+                # deleted remotely, we'll delete it locally.  Otherwise, we
+                # try to modify a deleted message's flags!  This step
+                # need only be taken if a statusfolder is present; otherwise,
+                # there is no action taken *to* the remote repository.
+
+                remotefolder.syncmessagesto_delete(localfolder, [localfolder,
+                                                                 statusfolder])
                 ui.syncingmessages(localrepos, localfolder, remoterepos, remotefolder)
                 localfolder.syncmessagesto(statusfolder, [remotefolder, statusfolder])
 
